

//#include <iostream>
#include "ConfigureStepsState.H"


using namespace avelino;
using namespace std;


ConfigureStepsState::ConfigureStepsState(Keypad &keypad, SevenSegments &sevenSegments, Stepper &stepper) : keypad(keypad), sevenSegments(sevenSegments), stepper(stepper), nextState(nullptr) {
    this->stepper.stop();
    this->configureButtonPressed = false;
    this->status = Status::STOPPED;
}


void ConfigureStepsState::stepDone(Stepper &s) {
    if (this->status == Status::RUNNING_COUNTERCLOCK)
        this->stepsCounter--;
    else if (this->status == Status::RUNNING_CLOCK)
        this->stepsCounter++;
    //cout << "BethlehemConfigureStepsState::stepDone: stepCounter=" << this->stepsCounter << endl;
}


void ConfigureStepsState::onLoad(SharedData &data) {
    this->data = &data;
    this->sevenSegments.setValue('c');
    this->stepper.setTicksPerStep(this->getTicksPerStep());
    this->stepper.setListener(*this);
    this->stepper.stop();
    this->stepsCounter = 0;
    this->configureButtonPressed = false;
    this->status = Status::STOPPED;
}


void ConfigureStepsState::run() {
    uint32_t k = this->keypad.getKeyMask();
    this->keypad.consume();
    if (this->status == Status::STOPPED) {
        if (k & Keypad::CONFIG_BUTTON)
            this->configureButtonPressed = true;
        else if (k & Keypad::DEC_BUTTON) {
            this->stepper.setWise(Stepper::Wise::COUNTERCLOCK);
            this->stepper.start();
            this->status = Status::RUNNING_COUNTERCLOCK;
            //cout << "BethlehemConfigureStepsState::run: counterclock" << endl;
        }
        else if (k & Keypad::INC_BUTTON) {
            this->stepper.setWise(Stepper::Wise::CLOCK);
            this->stepper.start();
            this->status = Status::RUNNING_CLOCK;
            //cout << "BethlehemConfigureStepsState::run: clock       " << endl;
        }
    }
    else if (this->status == Status::RUNNING_COUNTERCLOCK) {
        //cout << "BethlehemConfigureStepsState::run: x           " << endl;
        if (k & Keypad::DEC_BUTTON) {
            this->stepper.stop();
            this->status = Status::STOPPED;
            //cout << "BethlehemConfigureStepsState::run: stop        " << endl;
        }
    }
    else if (this->status == Status::RUNNING_CLOCK) {
        //cout << "BethlehemConfigureStepsState::run: x           " << endl;
        if (k & Keypad::INC_BUTTON) {
            this->stepper.stop();
            this->status = Status::STOPPED;
            //cout << "BethlehemConfigureStepsState::run: stop        " << endl;
        }
    }
}


State<SharedData> &ConfigureStepsState::getNextState() {
    if (this->configureButtonPressed)
        return *(this->nextState);
    else
        return *this;
}


SharedData &ConfigureStepsState::onUnload() {
    this->data->steps = this->stepsCounter;
    return *(this->data);
}


