

#include "IdleState.H"


using namespace avelino;
using namespace std;


IdleState::IdleState(Keypad &keypad, SevenSegments &sevenSegments, Stepper &stepper) : keypad(keypad), sevenSegments(sevenSegments), nextState(nullptr), stepperPathManager(stepper), configureButtonPressed(false) {
    this->stepperPathManager.stop();
}


void IdleState::onLoad(SharedData &data) {
    this->data = &data;
    uint32_t ticksToZero = this->data->days * this->getTicksPerDay();
    this->sevenSegments.setValue(' ');
    this->stepperPathManager.start(this->data->steps, ticksToZero);
    this->configureButtonPressed = false;
}


void IdleState::run() {
    uint8_t k = this->keypad.getKeyMask();
    this->keypad.consume();
    if (k & Keypad::CONFIG_BUTTON)
        this->configureButtonPressed = true;
    this->stepperPathManager.run();
}


State<SharedData> &IdleState::getNextState() {
    if (this->configureButtonPressed)
        return *(this->nextState);
    else
        return *this;
}


SharedData &IdleState::onUnload() {
    this->stepperPathManager.stop();
    return *(this->data);
}

