

#ifndef  __STEPPER_H__
#define  __STEPPER_H__


#include "Task.H"
#include <stdint.h>


extern "C++" {
    namespace avelino {
        using namespace std;

        class Stepper;

        class StepperListener {
            public:
                virtual void stepDone(Stepper &s) = 0;
        };

        class Stepper : public Task {
            public:
                enum class Wise { CLOCK, COUNTERCLOCK };
            protected:
                int32_t ticksPerStep;
                int32_t timer;
                Wise wise;
                enum class Status { STOPPED, RUNNING };
                Status status;
                bool doStart;
                uint32_t lastMask;
                StepperListener *listener;
            public:
                Stepper() : ticksPerStep(1), wise(Wise::CLOCK), status(Status::STOPPED), doStart(false), lastMask(1), listener(nullptr) { };
                void init();
                virtual void gpioConfigure() const = 0;
                virtual void gpioWrite(uint32_t mask) const = 0;
                virtual uint32_t getNextMask(uint32_t prevMask, Wise wise) const = 0;
                virtual const uint32_t getStepsPerRevolution() const = 0;
                virtual const uint32_t getTicksPerPulse() const = 0;
                void setListener(StepperListener &listener) { this->listener = &listener; };
                void start();
                void stop();
                virtual void run();
                void setWise(Wise w) { this->wise = w; };
                void setTicksPerStep(int32_t v) { this->ticksPerStep = v; };
        };
    }
}


#endif  // __STEPPER_H__
