

#include "Task.H"
#include "Timer.H"
#include "MyStepper.H"
#include "MyKeypad.H"
#include "MySevenSegments.H"
#include "SharedData.H"
#include "State.H"
#include "MyIdleState.H"
#include "MyConfigureStepsState.H"
#include "ConfigureDaysState.H"
#include "interrupt.H"
#include "Heartbeat.H"


using namespace avelino;
using namespace std;


class MyTimerListener : public TimerListener {
	public:
		Task **tasks;
		uint16_t numTasks;
		MyTimerListener(Task **tasks, uint16_t numTasks) : tasks(tasks), numTasks(numTasks) { };
		virtual void timerExpired();
};


void MyTimerListener::timerExpired() {
	Task **t = this->tasks;
	for (uint16_t i = 0; i < this->numTasks; i++, t++)
		(*t)->run();
}


int main() {
	interruptInit();
    MyKeypad keypad;
    keypad.init();
    MySevenSegments sevenSegments;
    sevenSegments.init();
    MyStepper stepper;
    stepper.init();
    Heartbeat heartbeat;
    // states
    SharedData data;
    MyConfigureStepsState configureStepsState(keypad, sevenSegments, stepper);
    ConfigureDaysState configureDaysState(keypad, sevenSegments, stepper);
    MyIdleState idleState(keypad, sevenSegments, stepper);
    // link states
    configureStepsState.setNextState(configureDaysState);
    configureDaysState.setNextState(idleState);
    idleState.setNextState(configureStepsState);
    StateManager<SharedData> stateManager(configureStepsState, data);
    // round robin tasks
	const int NUM_TASKS = 5;
	Task *tasks[NUM_TASKS] = { &keypad, &sevenSegments, &stepper, &heartbeat, &stateManager };
	MyTimerListener myTimerListener(tasks, NUM_TASKS);
	Timer timer;
	timer.init(myTimerListener, 10_msForTimer);
	while (true)
		asm volatile ("wfi");
    return 0;
}
